package com.foreverwin.mesnac.integration.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.constant.IntegrationTypeConstant;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.integration.model.IntegrationLog;
import com.foreverwin.mesnac.integration.model.WebServiceDto;
import com.foreverwin.mesnac.integration.service.ERPWebService;
import com.foreverwin.mesnac.integration.service.IntegrationLogService;
import com.foreverwin.mesnac.integration.service.InterfaceService;
import com.foreverwin.modular.core.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.jws.WebParam;
import javax.jws.WebService;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Service
@WebService(serviceName = "ERPWebService",
            targetNamespace = "http://service.integration.mesnac.foreverwin.com/",
            endpointInterface = "com.foreverwin.mesnac.integration.service.ERPWebService")
public class ERPWebServiceImpl implements ERPWebService {

    private static final Logger logger = LoggerFactory.getLogger(ERPWebService.class);


    @Autowired
    private InterfaceService interfaceService;
    @Autowired
    private IntegrationLogService integrationLogService;

    /**
     * {
     *  "TRANID": "2021052616434400028",
     *  "DATE_TIME": "2021-06-16 15:31:00",
     *  "SITE": "1000",
     *  "ITEM": "12053-000170A",
     *  "REVISION": "A",
     *  "MAKTX": "活扳手18\"(250mm) 87-371-1-23",
     *  "STATUS": "201",
     *  "ITEM_TYPE": "ROH",
     *  "PROCUREMENT_TYPE": "P",
     *  "LOT_SIZE": "9999999",
     *  "ASSY_DATA_TYPE": "",
     *  "INV_ASSY_DATA_TYPE": "",
     *  "UNIT_OF_MEASURE": "KG"
     * }
     *
     * @param data
     * @return
     */
    @Override
    public String createItem(@WebParam(name = "data") String data) {
        logger.info("接收ERP传递物料参数:" + data);

        String site = null;
        String handle = null;
        String dateTime = null;
        JSONObject jsonObject = null;
        WebServiceDto webServiceDto = WebServiceDto.success("物料保存/更新成功");
        try {
            jsonObject = JSON.parseObject(data);
            site = jsonObject.getString("SITE");
            handle = jsonObject.getString("TRANID");
            dateTime = jsonObject.getString("DATE_TIME");
            if (StringUtil.isBlank(site)) {
                throw BusinessException.build("站点参数不能为空");
            }
            if (StringUtil.isBlank(dateTime)) {
                throw BusinessException.build("传送时间不能为空");
            }

            interfaceService.createOrUpdateItem(jsonObject);
        } catch (Exception e) {
            webServiceDto = WebServiceDto.fail(handle, e.getMessage());
            return JSONObject.toJSONString(webServiceDto);
        }

        //记录接口日志
        IntegrationLog integrationLog = new IntegrationLog();
        integrationLog.setHandle(handle);
        integrationLog.setSite(site);
        integrationLog.setIntegrationType(IntegrationTypeConstant.ITEM);
        integrationLog.setCategory("REQUEST");
        integrationLog.setIntegrationWay("ERP");
        integrationLog.setIntegrationMethod("createOrUpdateItem");
        integrationLog.setParam(jsonObject.toJSONString());
        integrationLog.setStatus(webServiceDto.getSTATUS());
        integrationLog.setResultMessage(webServiceDto.getMESSAGE());

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        integrationLog.setRequestDateTime(LocalDateTime.parse(dateTime, formatter));
        integrationLog.setTransactionId(handle);
        integrationLog.setCreatedDateTime(LocalDateTime.now());
        integrationLogService.saveIntegrationLog(integrationLog);

        //接口返回
        return JSONObject.toJSONString(webServiceDto);
    }


    /**
     * {
     *  "TRANID": "2021052616434400028",
     *  "DATE_TIME": "2021-06-16 15:31:00",
     *  "SITE": "1000",
     *  "FACTORY": "",
     *  "PURCHASE_ORDER": "",  //采购订单
     *  "PURCHASE_ORDER_LIST": [{
     *      "SEQ": "",    //行号
     *      "ITEM": "",   //物料
     *      "DESCRIPTION": "",
     *      "SPECIFICATION": "",
     *      "SURFACES": "",
     *      "BATCH": "",  //批次【供应商代码+接收日期（年月日）】
     *      "SUPPLIER": ""
     *  }]
     * }
     *
     * @param data
     * @return
     */
    @Override
    public String itemBatchReceive(@WebParam(name = "data") String data) {
        logger.info("接收ERP传递物料批次接收参数:" + data);

        String site = null;
        String handle = null;
        String dateTime = null;
        JSONObject jsonObject = null;
        WebServiceDto webServiceDto = WebServiceDto.success("物料批次接收成功");
        try {
            jsonObject = JSON.parseObject(data);
            site = jsonObject.getString("SITE");
            handle = jsonObject.getString("TRANID");
            dateTime = jsonObject.getString("DATE_TIME");
            if (StringUtil.isBlank(site)) {
                throw BusinessException.build("站点参数不能为空");
            }
            if (StringUtil.isBlank(dateTime)) {
                throw BusinessException.build("传送时间不能为空");
            }

            interfaceService.saveItemBatch(jsonObject);
        } catch (Exception e) {
            webServiceDto = WebServiceDto.fail(handle, e.getMessage());
            return JSONObject.toJSONString(webServiceDto);
        }

        //记录接口日志
        IntegrationLog integrationLog = new IntegrationLog();
        integrationLog.setHandle(handle);
        integrationLog.setSite(site);
        integrationLog.setIntegrationType(IntegrationTypeConstant.ITEM_BATCH_RECEIVE);
        integrationLog.setCategory("REQUEST");
        integrationLog.setIntegrationWay("ERP");
        integrationLog.setIntegrationMethod("saveItemBatch");
        integrationLog.setParam(jsonObject.toJSONString());
        integrationLog.setStatus(webServiceDto.getSTATUS());
        integrationLog.setResultMessage(webServiceDto.getMESSAGE());

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        integrationLog.setRequestDateTime(LocalDateTime.parse(dateTime, formatter));
        integrationLog.setTransactionId(handle);
        integrationLog.setCreatedDateTime(LocalDateTime.now());
        integrationLogService.saveIntegrationLog(integrationLog);

        //接口返回
        return JSONObject.toJSONString(webServiceDto);
    }
}
